@@ -28,6 +28,8 @@
 from __future__ import print_function
 
 import os
+import sys
+import cStringIO
 
 from rospkg import RosPack, RosStack
 
@@ -420,14 +422,24 @@ def test_RosdepInstaller_get_uninstalled():
 
         expected = set(['libltdl-dev', 'libboost1.40-all-dev', 'libtool'])
         uninstalled, errors = installer.get_uninstalled(['roscpp_fake'], verbose)
-        assert uninstalled.keys() == [APT_INSTALLER]
-        assert set(uninstalled[APT_INSTALLER]) == expected
+        keys, values = zip(*uninstalled)
+        apt_uninstalled = []
+        for k, v in uninstalled:
+            if k == APT_INSTALLER:
+                apt_uninstalled.extend(v)
+        assert list(set(keys)) == [APT_INSTALLER]
+        assert set(apt_uninstalled) == expected
         assert not errors
 
         expected = ['libtinyxml-dev']
         uninstalled, errors = installer.get_uninstalled(['rospack_fake'], verbose)
-        assert uninstalled.keys() == [APT_INSTALLER]
-        assert uninstalled[APT_INSTALLER] == expected, uninstalled
+        keys, values = zip(*uninstalled)
+        apt_uninstalled = []
+        for k, v in uninstalled:
+            if k == APT_INSTALLER:
+                apt_uninstalled.extend(v)
+        assert list(set(keys)) == [APT_INSTALLER]
+        assert apt_uninstalled == expected, uninstalled
         assert not errors
 
 def get_fake_apt(detect_fn):
@@ -495,7 +507,7 @@ def test_RosdepInstaller_get_uninstalled_unconfigured():
     # annoying mock to test generally impossible error condition
     from mock import Mock
     lookup = Mock(spec=RosdepLookup)
-    lookup.resolve_all.return_value = ({'bad-key': ['stuff']}, [])
+    lookup.resolve_all.return_value = ([('bad-key', ['stuff'])], [])
     
     installer = RosdepInstaller(context, lookup)
     try:
@@ -503,3 +515,49 @@ def test_RosdepInstaller_get_uninstalled_unconfigured():
         assert False, "should have raised"
     except RosdepInternalError:
         pass
+
+
+from contextlib import contextmanager
+@contextmanager
+def fakeout():
+    realstdout = sys.stdout
+    realstderr = sys.stderr
+    fakestdout = cStringIO.StringIO()
+    fakestderr = cStringIO.StringIO()
+    sys.stdout = fakestdout
+    sys.stderr = fakestderr
+    yield fakestdout, fakestderr
+    sys.stdout = realstdout
+    sys.stderr = realstderr
+
+def test_RosdepInstaller_install_resolved():
+    from rosdep2 import create_default_installer_context
+    from rosdep2.lookup import RosdepLookup
+    from rosdep2.installers import RosdepInstaller
+    from rosdep2.platforms.debian import APT_INSTALLER
+    
+    from rosdep2.lookup import RosdepLookup
+    rospack, rosstack = get_test_rospkgs()
+    
+    # create our test fixture.  use most of the default toolchain, but
+    # replace the apt installer with one that we can have more fun
+    # with.  we will do all tests with ubuntu lucid keys -- other
+    # tests should cover different resolution cases.
+    sources_loader = create_test_SourcesListLoader()
+    lookup = RosdepLookup.create_from_rospkg(rospack=rospack, rosstack=rosstack, sources_loader=sources_loader)
+    context = create_default_installer_context()
+    context.set_os_override('ubuntu', 'lucid')
+    installer = RosdepInstaller(context, lookup)
+    
+    with fakeout() as (stdout, stderr):
+        installer.install_resolved(APT_INSTALLER, [], simulate=True, verbose=False)
+    with fakeout() as (stdout, stderr):
+        installer.install_resolved(APT_INSTALLER, [], simulate=True, verbose=True)
+    assert stdout.getvalue().strip() == '#No packages to install'
+    with fakeout() as (stdout, stderr):            
+        installer.install_resolved(APT_INSTALLER, ['rosdep-fake1', 'rosdep-fake2'], simulate=True, verbose=True)
+    stdout_lines = [x.strip() for x in stdout.getvalue().split('\n') if x.strip()]
+    assert stdout_lines == ['#[apt] Installation commands:',
+                            'sudo apt-get install rosdep-fake1',
+                            'sudo apt-get install rosdep-fake2',
+                            ], ("%s: %s"%(stdout.getvalue(), stdout_lines))
