@@ -29,12 +29,24 @@
 
 import os
 import traceback
-from mock import patch
+from mock import patch, Mock
 
 def get_test_dir():
     # not used yet
     return os.path.abspath(os.path.join(os.path.dirname(__file__), 'redhat'))
 
+def test_rpm_detect():
+    from rosdep2.platforms.redhat import rpm_detect
+
+    m = Mock()
+    m.return_value = ''
+
+    val = rpm_detect([], exec_fn=m)
+    assert val == [], val
+
+    val = rpm_detect(['tinyxml-dev'], exec_fn=m)
+    assert val == [], val
+
 def test_YumInstaller():
     from rosdep2.platforms.redhat import YumInstaller
 
@@ -59,4 +71,3 @@ def test_YumInstaller():
     except AssertionError:
         traceback.print_exc()
         raise
-    
